<?php
/**
Functions for Ship-Quik
 **/

namespace ShipQuik\WooCommerce;

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly.
}

use \ShipQuik\Shared\ShipQuik_Suppliers;
use \ShipQuik\Shared\ShipQuik_PricingMode;
use \ShipQuik\Shared\ShipQuik_ShippingTaxes;

use WC_Shipping_Rates;
use WC_Shipping_Rate;

class ShipQuik_ShippingRate
{
    protected $parent;

    protected $config;

    protected $shippingPackage;

    protected $rates;

    public $rate;

    public function __construct($parent, $rates, $shippingPackage)
    {
        $this->parent           = $parent;
        $this->rates            = $rates;
        $this->shippingPackage  = $shippingPackage;

        $this->config           = $parent->config;
    }

    private function isValidDestination() {
        $customerAddress = $this->shippingPackage->getDestination();

        if ($customerAddress['country'] && $customerAddress['postcode']) return true;

        return false;
    }

    /**
     * @return array
     */
    public function getRates()
    {
        if (!$this->isValidDestination()) return $this->rates;

        $shiQuikSuppliers = new ShipQuik_Suppliers($this->parent, $this->shippingPackage->getBoxes($this->config, $this->shippingPackage->getProducts()), $this->shippingPackage->getDestination());

        $pricingMode = ShipQuik_PricingMode::create($this->parent, $this->config->getValueByKey('pricingMode'))->getForRates($shiQuikSuppliers);

        if (empty($pricingMode->packages)) {
            return $this->removeShipQuikRates($this->rates);
        }

        foreach ($this->rates as $key => $rate) {
            if (self::isShipQuik($rate)) {
                $this->rate = $rate;
                $result = $pricingMode->getModeRate($this);

                if ($result) {
                    $isFree = $this->isFreeShipping($rate);

                    // If is freeShipping, not continue
                    if ($isFree) {
                        $freeRate = new WC_Shipping_Rate();
                        $freeRate->set_id($rate->get_id());
                        $freeRate->set_method_id($rate->get_method_id());
                        $freeRate->set_instance_id($rate->get_instance_id());
                        $freeRate->set_label(__('Free shipping', 'ship-quik'));
                        $freeRate->set_cost(0);
                        $freeRate->set_taxes(0);

                        $freeRate->add_meta_data('_supplier_data', json_encode($result, JSON_UNESCAPED_UNICODE));
                        $freeRate->add_meta_data('Supplier', __($result['supplier'], 'ship-quik'));
                        $freeRate->add_meta_data('Servicio', __($result['service'], 'ship-quik'));
                        $freeRate->add_meta_data('_logo', $result['supplier']);

                        $rates[$rate->get_id()] = $freeRate;

                        return $rates;
                    }

                    // If has tableRate
                    $tableRatePrice = $this->getShipQuikTableRate();
                    if ($tableRatePrice > 0) {
                        $tableRate = new WC_Shipping_Rate();
                        $tableRate->set_id($rate->get_id());
                        $tableRate->set_method_id($rate->get_method_id());
                        $tableRate->set_instance_id($rate->get_instance_id());
                        $tableRate->set_label(__('with Ship-Quik', 'ship-quik'));
                        $tableRate->set_cost($tableRatePrice);
                        $tableRate->set_taxes(0);

                        $rates[$rate->get_id()] = $tableRate;

                        return $rates;
                    }

                    $cost = $this->getShopFee($result['salesPrice']);

                    if ($rate->get_shipping_tax()) {
                        $rate->set_cost($cost);
                    } else {
                        $customerAddress = $this->shippingPackage->getDestination();

                        $taxesInfo = [
                            "countryCodeOrigin"      => $this->config->getValueByKey('senderAddressCountryCode'),
                            "countryCodeDestination" => $customerAddress['country'],
                            "postalCodeOrigin"       => $this->config->getValueByKey('senderAddressPostalCode'),
                            "postalCodeDestination"  => $customerAddress['postcode'],
                            "idUser"                 => $this->config->getIdCliente()
                        ];
                        $shippingTaxes = new ShipQuik_ShippingTaxes($this->parent, $taxesInfo);
                        $resultTaxes = $shippingTaxes->send();
                
                        $iva = 0;
                        if ($resultTaxes) {
                            $iva = $cost * ($resultTaxes['ivaInfo']['percentage'] / 100);
                        }
                        $rate->set_cost($cost + $iva);
                    };

                    $rate->add_meta_data('_supplier_data', json_encode($result, JSON_UNESCAPED_UNICODE));
                    $rate->add_meta_data('Supplier', __($result['supplier'], 'ship-quik'));
                    $rate->add_meta_data('Servicio', __($result['service'], 'ship-quik'));
                    $rate->add_meta_data('_logo', $result['supplier']);
                }
            }
        }

        return $this->rates;
    }

    private function removeShipQuikRates($rates)
    {
        $resultRates = [];

        foreach ($this->rates as $key => $rate) {
            if (!self::isShipQuik($rate)) {
                $resultRates[$key] = $rate;
            }
        }

        return $resultRates;
    }

    public static function isShipQuik($rate)
    {
        $shippingSupplierService = explode('-', $rate->get_method_id());

        if ($shippingSupplierService[0] == 'shipquik') {
            return true;
        }
        return false;
    }

    public function getSupplier($rate)
    {
        $shippingSupplierService = explode('-', $rate->get_method_id());

        if (isset($shippingSupplierService[1])) {
            return $shippingSupplierService[1];
        }

        return false;
    }

    public function getService($rate)
    {
        $shippingSupplierService = explode('-', $rate->get_method_id());

        if (isset($shippingSupplierService[2])) {
            return strtoupper($shippingSupplierService[2]);
        }

        return false;
    }

    private function isFreeShipping($rate)
    {
        if ($this->config->getValueByKey('isFreeShipping')) {
            return true;
        }

        return false;
    }

    private function hasShopFeeFrom()
    {
        $total = floatval(WC()->cart->cart_contents_total);

        if (floatval($this->config->getValueByKey('freeShippingPriceFrom')) > 0 && $total >= floatval($this->config->getValueByKey('freeShippingPriceFrom'))) {
            return true;
        }

        return false;
    }

    private function getShopFee($price)
    {
        if ($this->hasShopFeeFrom()) return $price;

        $shopFeeValue = floatval($this->config->getValueByKey('shopFeeValue'));
        $shopFeeType  = $this->config->getValueByKey('shopFeeType');

        $price = floatval($price);

        if ($shopFeeValue > 0) {
            if ($shopFeeType== 'price') {
                $price = $price + $shopFeeValue;
            } else {
                $price = $price + ($price * $shopFeeValue / 100);
            }
        }

        return $price;
    }

    private function getShipQuikTableRate()
    {
        $resultRates = 0;

        $total = floatval(WC()->cart->cart_contents_total);

        $tableRates = $this->config->getValueByKey('shippingTableRates');

        if (!$tableRates) {
            $tableRates = [];
        } else {
            $tableRates = unserialize($tableRates);
        }

        if (!empty($tableRates)) {
            foreach ($tableRates as $rate) {
                if ($total >= intval($rate['from']) && $total <= intval($rate['to'])) {
                    $resultRates = floatval($rate['value']);
                }
            }
        }

        return $resultRates;
    }
}
